import pandas as pd
import numpy as np
import os

from scipy.stats import mannwhitneyu


def compute_median(df):
    grp = df.groupby(['Framework', 'Problem'])

    median = grp.median()

    q1 = grp['R2 train'].quantile(0.25)
    q3 = grp['R2 train'].quantile(0.75)
    median['R2 train IQR'] = q3 - q1

    q1 = grp['R2 test'].quantile(0.25)
    q3 = grp['R2 test'].quantile(0.75)
    median['R2 test IQR'] = q3 - q1

    q1 = grp['RMSE train'].quantile(0.25)
    q3 = grp['RMSE train'].quantile(0.75)
    median['RMSE train IQR'] = q3 - q1

    q1 = grp['RMSE test'].quantile(0.25)
    q3 = grp['RMSE test'].quantile(0.75)
    median['RMSE test IQR'] = q3 - q1

    q1 = grp['NMSE train'].quantile(0.25)
    q3 = grp['NMSE train'].quantile(0.75)
    median['NMSE train IQR'] = q3 - q1

    q1 = grp['NMSE test'].quantile(0.25)
    q3 = grp['NMSE test'].quantile(0.75)
    median['NMSE test IQR'] = q3 - q1

    print(median[['NMSE train', 'NMSE train IQR', 'NMSE test', 'NMSE test IQR', 'Elapsed']].sort_values(by=['Problem', 'Framework'],axis='index').round(3).to_markdown())


def color_cell(val):
    color = 'red' if val < 0 else 'black'
    return 'color: %s' % color


# statistical testing
def statistical_test(df):
    problems = df.groupby(['Problem'])

    for problem_name, problem in problems:
        print(problem_name)

        algorithms = problem.groupby(['Framework'])

        names = list(algorithms.groups.keys())

        dim = len(algorithms)
        p_values_train = pd.DataFrame(np.zeros((dim, dim)))
        p_values_test = pd.DataFrame(np.zeros((dim, dim)))

        median_comp_train = pd.DataFrame(np.zeros((dim, dim)))
        median_comp_test = pd.DataFrame(np.zeros((dim, dim)))

        i = 0
        for a1, b1 in algorithms:
            p_values_train.loc[i, i]    = np.nan
            p_values_test.loc[i, i]     = np.nan
            median_comp_train.loc[i, i] = np.nan
            median_comp_test.loc[i, i]  = np.nan
            j = 0
            for a2, b2 in algorithms:
                if a1 != a2:
                    x1 = b1['NMSE train'].values
                    x2 = b2['NMSE train'].values

                    m1 = np.median(x1)
                    m2 = np.median(x2)

                    median_comp_train.loc[i, j] = True if m1 < m2 else False

                    r = mannwhitneyu(x1, x2)

                    p_values_train.loc[i, j] = p_values_train.loc[j, i] = r.pvalue

                    x1 = b1['NMSE test'].values
                    x2 = b2['NMSE test'].values

                    m1 = np.median(x1)
                    m2 = np.median(x2)

                    median_comp_test.loc[i, j] = True if m1 < m2 else False

                    r = mannwhitneyu(x1, x2)

                    p_values_test.loc[i, j] = p_values_test.loc[j, i] = r.pvalue
                j += 1
            i += 1

        #median_comp_train.style.applymap(color_cell)
        #print(median_comp_train)
        s = p_values_train.where(median_comp_train).style.precision(6).applymap(lambda x: 'bg: red' if x < 0.05 else 'color: black', axis=None)           
        print(s)
        #print(p_values_train.round(5))
        #df_train.to_csv('{}_train.csv'.format(problem_name))

        #print(p_values_test.round(5))
        #df_test.to_csv('{}_test.csv'.format(problem_name))


if __name__ == '__main__':
    df = pd.read_csv('./aggregated_results.csv', sep=',')

    statistical_test(df)
